-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/cleanupdistributedresources.sql
-- SID: 1.3
-- Date Modified: 04/01/09 14:47:34
-- Procedure used to cleanup distributed resources

USE Object
GO

---------------------------------------------------------------------------------------------------
-- CleanupDistributedResources
PRINT 'Creating procedure CleanupDistributedResources'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'CleanupDistributedResources' AND type = 'P')
  DROP PROCEDURE CleanupDistributedResources
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/cleanupdistributedresources.sql
-- SID: 1.3
-- Date Modified: 04/01/09 14:47:34
-- Procedure used to cleanup distributed resources
CREATE PROCEDURE CleanupDistributedResources
  @cid          ClassID = 'ROOT',
  @id           ObjID = 1,
  @verbose      INT = 0
AS
  SET NOCOUNT ON

  DECLARE @err INT, @rc INT, @now NVARCHAR(40)

  -- If there is no outer transaction then begin a transaction, otherwise create
  -- a transaction savepoint
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION CleanupDistributedResources
  ELSE
    SAVE TRANSACTION CleanupDistributedResources

  -- cleanup dup objects
  CREATE TABLE #objs
  (
    cid    	ClassID 	NULL,
    id     	ObjID 		NULL,
    resource 	NVARCHAR(255) 	NULL 
  )

  --Insert NODE resources that are duplicate and have a higher id into #objs
  INSERT INTO #objs (cid, id, resource)
    SELECT DISTINCT 'NODE', pri.id, pri.name
    FROM NetworkNode_V AS pri
    JOIN NetworkNode_V AS sec ON sec.name = pri.name 
    WHERE  
      sec.deleted = 0 AND 
      pri.deleted = 0 AND
      pri.id > sec.id AND
      pri.id NOT IN (SELECT oid FROM CL_IDCache WHERE cid = 'NODE') 
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Error inserting NODE resources into #objs. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION CleanupDistributedResources
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Inserted %d NODE resources into #objs.', 0, 1, @now, @rc) WITH NOWAIT
  END

  --Insert UNIX resources that are duplicate and have a higher id into #objs
  INSERT INTO #objs (cid, id, resource)
    SELECT DISTINCT 'UNIX', pri.id, pri.name
    FROM NetworkUnix_V AS pri
    JOIN NetworkUnix_V AS sec ON sec.name = pri.name 
    WHERE  
      sec.deleted = 0 AND 
      pri.deleted = 0 AND
      pri.id > sec.id AND
      pri.id NOT IN (SELECT oid FROM CL_IDCache WHERE cid = 'UNIX') 
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Error inserting UNIX resources into #objs. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION CleanupDistributedResources
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Inserted %d UNIX resources into #objs.', 0, 1, @now, @rc) WITH NOWAIT
  END

  --Insert NTSR resources that are duplicate and have a higher id into #objs
  INSERT INTO #objs (cid, id, resource)
    SELECT DISTINCT 'NTSR', pri.id, pri.name
    FROM NTServer_V AS pri
    JOIN NTServer_V AS sec ON sec.name = pri.name 
    WHERE  
      sec.deleted = 0 AND 
      pri.deleted = 0 AND
      pri.id > sec.id AND
      pri.id NOT IN (SELECT oid FROM CL_IDCache WHERE cid = 'NTSR') 
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Error inserting NTSR resources into #objs. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION CleanupDistributedResources
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Inserted %d NTSR resources into #objs.', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Delete if they dont have G0% object as children
  DELETE ob
    FROM #objs AS ob
    JOIN link AS l ON ob.cid = l.src_cid AND ob.id = l.src_id
    WHERE l.deleted = 0 AND l.dst_cid NOT IN (
      SELECT cid FROM GEMLookupCID
    ) AND NOT EXISTS (SELECT * FROM link WHERE ob.cid = l.src_cid AND ob.id = l.src_id AND link_type = 'GEPC')
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Error deleting resources not having GO% resources as children. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION CleanupDistributedResources
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Deleted %d resources not having G0% resources as children.', 0, 1, @now, @rc) WITH NOWAIT
  END

  IF NOT EXISTS (SELECT * FROM #objs) 
    RAISERROR('No resources to delete.', 0, 1) WITH NOWAIT

  DECLARE @path NVARCHAR(1024), @name NVARCHAR(255)
  -- NODE  -  delete invalid resource that was created.
  DECLARE deletenode INSENSITIVE CURSOR FOR
    SELECT t.id FROM #objs AS t WHERE t.cid = 'NODE'
  OPEN deletenode
  DECLARE @node_id ObjID
  FETCH NEXT FROM deletenode INTO @node_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @path = resourcepath, @name = resource FROM ObjPathCache WHERE cid = 'NODE' AND id = @node_id
    IF @path IS NOT NULL
      RAISERROR('Deleting resource %s with path %s', 0, 1, @name, @path) WITH NOWAIT
    ELSE 
      RAISERROR('Deleting resource with CID=NODE, ID=%d.  Unable to determine resource path.', 0, 1, @node_id) WITH NOWAIT
    EXEC delete_NODE @node_id
    SELECT @err = @@ERROR
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: CleanupDistributedResources: Error deleting NODE resource. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      DEALLOCATE deletenode
      ROLLBACK TRANSACTION CleanupDistributedResources
      RETURN -30000
    END
    DELETE FROM #objs WHERE cid = 'NODE' and id = @node_id
    FETCH NEXT FROM deletenode INTO @node_id
  END
  DEALLOCATE deletenode

  -- UNIX  -  delete invalid resource that was created.
  DECLARE deleteunix INSENSITIVE CURSOR FOR
    SELECT t.id FROM #objs AS t WHERE t.cid = 'UNIX'
  OPEN deleteunix
  DECLARE @unix_id ObjID
  FETCH NEXT FROM deleteunix INTO @unix_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @path = resourcepath, @name = resource FROM ObjPathCache WHERE cid = 'UNIX' AND id = @unix_id
    IF @path IS NOT NULL
      RAISERROR('Deleting resource %s with path %s', 0, 1, @name, @path) WITH NOWAIT
    ELSE 
      RAISERROR('Deleting resource with CID=UNIX, ID=%d.  Unable to determine resource path.', 0, 1, @unix_id) WITH NOWAIT
    EXEC delete_UNIX @unix_id
    SELECT @err = @@ERROR
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: CleanupDistributedResources: Error deleting UNIX resource. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      DEALLOCATE deleteunix
      ROLLBACK TRANSACTION CleanupDistributedResources
      RETURN -30000
    END
    DELETE FROM #objs WHERE cid = 'UNIX' and id = @unix_id
    FETCH NEXT FROM deleteunix INTO @unix_id
  END
  DEALLOCATE deleteunix

  -- NTSR  -  delete invalid resource that was created.
  DECLARE deletentsr INSENSITIVE CURSOR FOR
    SELECT t.id FROM #objs AS t WHERE t.cid = 'NTSR'
  OPEN deletentsr
  DECLARE @ntsr_id ObjID
  FETCH NEXT FROM deletentsr INTO @ntsr_id
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SELECT @path = resourcepath, @name = resource FROM ObjPathCache WHERE cid = 'NTSR' AND id = @ntsr_id
    IF @path IS NOT NULL
      RAISERROR('Deleting resource %s with path %s', 0, 1, @name, @path) WITH NOWAIT
    ELSE 
      RAISERROR('Deleting resource with CID=NTSR, ID=%d.  Unable to determine resource path.', 0, 1, @ntsr_id) WITH NOWAIT
    EXEC delete_NTSR @ntsr_id
    SELECT @err = @@ERROR
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: CleanupDistributedResources: Error deleting NTSR resource. Error=%d', 3, 1, @now, @err) WITH NOWAIT
      DEALLOCATE deletentsr
      ROLLBACK TRANSACTION CleanupDistributedResources
      RETURN -30000
    END
    DELETE FROM #objs WHERE cid = 'NTSR' and id = @ntsr_id
    FETCH NEXT FROM deletentsr INTO @ntsr_id
  END
  DEALLOCATE deletentsr

  --Update link_type and deleted flag for link and individual link tables
  CREATE TABLE #lnks  
  (
    src_cid    	ClassID 	NULL,
    src_id     	ObjID 		NULL,
    dst_cid	ClassID		NULL,
    dst_id	ObjID		NULL,
    link_type	NVARCHAR(4)	NULL
  )

  -- Insert NODE resources into temp table
  INSERT INTO #lnks
    SELECT DISTINCT l.src_cid, l.src_id, l.dst_cid, n.id, l.link_type 
    FROM NetworkNode_V AS n
    JOIN link AS l ON l.dst_id = n.id 
    JOIN link AS l2 ON l2.src_id = l.dst_id
    JOIN NLOC_NODE_PHYC AS nloc ON nloc.dst_id = l.dst_id AND nloc.src_id = l.src_id
  WHERE 
    n.id NOT IN (SELECT oid FROM CL_IDCache WHERE cid = 'NODE') 
    AND n.deleted = 0
    AND l2.src_cid = 'NODE'
    AND l2.dst_cid IN (SELECT cid FROM GEMLookupCID)
    AND l.src_cid = 'NLOC' 
    AND l.dst_cid = 'NODE'
    AND l.deleted = 0
    AND l.link_type = 'GEPC'
    AND nloc.deleted = 1
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Error inserting NODE resources into #lnks. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION CleanupDistributedResources
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Inserted %d NODE resources into #lnks.', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Insert UNIX resources into temp table
  INSERT INTO #lnks
    SELECT DISTINCT l.src_cid, l.src_id, l.dst_cid, n.id, l.link_type 
    FROM NetworkUnix_V AS n
    JOIN link AS l ON l.dst_id = n.id 
    JOIN link AS l2 ON l2.src_id = l.dst_id
    JOIN NLOC_UNIX_PHYC AS nloc ON nloc.dst_id = l.dst_id AND nloc.src_id = l.src_id
  WHERE 
    n.id NOT IN (SELECT oid FROM CL_IDCache WHERE cid = 'UNIX') 
    AND n.deleted = 0
    AND l2.src_cid = 'UNIX'
    AND l2.dst_cid IN (SELECT cid FROM GEMLookupCID)
    AND l.src_cid = 'NLOC' 
    AND l.dst_cid = 'UNIX'
    AND l.deleted = 0
    AND l.link_type = 'GEPC'
    AND nloc.deleted = 1
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Error inserting UNIX resources into #lnks. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION CleanupDistributedResources
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Inserted %d UNIX resources into #lnks.', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Insert NTSR resources into temp table
  INSERT INTO #lnks
    SELECT DISTINCT l.src_cid, l.src_id, l.dst_cid, n.id, l.link_type 
    FROM NTServer_V AS n
    JOIN link AS l ON l.dst_id = n.id 
    JOIN link AS l2 ON l2.src_id = l.dst_id
    JOIN NLOC_NTSR_PHYC AS nloc ON nloc.dst_id = l.dst_id AND nloc.src_id = l.src_id
  WHERE 
    n.id NOT IN (SELECT oid FROM CL_IDCache WHERE cid = 'NTSR') 
    AND n.deleted = 0
    AND l2.src_cid = 'NTSR'
    AND l2.dst_cid IN (SELECT cid FROM GEMLookupCID)
    AND l.src_cid = 'NLOC' 
    AND l.dst_cid = 'NTSR'
    AND l.deleted = 0
    AND l.link_type = 'GEPC'
    AND nloc.deleted = 1
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Error inserting NTSR resources into #lnks. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION CleanupDistributedResources
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Inserted %d NTSR resources into #lnks.', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Change link type in link table to PHYC (from GEPC)
  UPDATE l SET l.link_type = 'PHYC'
    FROM link AS l
    JOIN #lnks AS tl ON l.src_cid = tl.src_cid AND l.src_id = tl.src_id AND l.dst_cid = tl.dst_cid AND l.dst_id = tl.dst_id
    WHERE l.link_type = 'GEPC' AND tl.link_type = 'GEPC'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Error updating link_type to PHYC in link table. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION CleanupDistributedResources
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Update link_type to PHYC for %d records in the link table.', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Insert into the containment table entries that should be there
  INSERT INTO containment
    SELECT DISTINCT tl.dst_cid, tl.dst_id, tl.src_cid, tl.src_id, 0
    FROM #lnks AS tl
  WHERE NOT EXISTS 
    (SELECT * FROM containment AS c 
     WHERE tl.dst_cid = c.dst_cid
       AND tl.dst_id  = c.dst_id
       AND tl.src_cid = c.src_cid
       AND tl.src_id  = c.src_id)
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Error inserting into containment table. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION CleanupDistributedResources
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Inserted %d records into containment table.', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Set deleted flag of PHYC to 0 in NLOC_NODE_PHYC
  UPDATE lnode SET lnode.deleted = 0
    FROM NLOC_NODE_PHYC AS lnode
    JOIN #lnks AS tl ON tl.src_id = lnode.src_id AND tl.dst_id = lnode.dst_id
    WHERE tl.src_cid = 'NLOC' AND tl.dst_cid = 'NODE' AND lnode.deleted = 1
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Error updating deleted flag to 0 in NLOC_NODE_PHYC table. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION CleanupDistributedResources
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Updated deleted flag to 0 for %d records in NLOC_NODE_PHYC table.', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Set deleted flag of PHYC to 0 in NLOC_UNIX_PHYC
  UPDATE lunix SET lunix.deleted = 0
    FROM NLOC_UNIX_PHYC AS lunix
    JOIN #lnks AS tl ON tl.src_id = lunix.src_id AND tl.dst_id = lunix.dst_id
    WHERE tl.src_cid = 'NLOC' AND tl.dst_cid = 'UNIX' AND lunix.deleted = 1
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Error updating deleted flag to 0 in NLOC_UNIX_PHYC table. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION CleanupDistributedResources
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Updated deleted flag to 0 for %d records in NLOC_UNIX_PHYC table.', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Set deleted flag of PHYC to 0 in NLOC_NTSR_PHYC
  UPDATE lntsr SET lntsr.deleted = 0
    FROM NLOC_NTSR_PHYC AS lntsr
    JOIN #lnks AS tl ON tl.src_id = lntsr.src_id AND tl.dst_id = lntsr.dst_id
    WHERE tl.src_cid = 'NLOC' AND tl.dst_cid = 'NTSR' AND lntsr.deleted = 1
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Error updating deleted flag to 0 in NLOC_NTSR_PHYC table. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION CleanupDistributedResources
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: CleanupDistributedResources: Updated deleted flag to 0 for %d records in NLOC_NTSR_PHYC table.', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- If we're the outer transaction then commit
  IF @tcount = 0
    COMMIT TRANSACTION CleanupDistributedResources
GO
